@@ -4,9 +4,9 @@
 
                  Copyright (C) 2000 NetCitadel, LLC
 
-  Author:  Vadim Kurland     vadim@vk.crocodile.org
+  Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: AddressRange.h 65 2008-05-20 02:25:49Z vadim $
+  $Id: AddressRange.h 236 2008-12-25 20:36:41Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -29,6 +29,7 @@
 
 #include <fwbuilder/Address.h>
 #include <fwbuilder/InetAddr.h>
+#include <fwbuilder/ObjectMatcher.h>
 
 namespace libfwbuilder
 {
@@ -43,7 +44,7 @@ class AddressRange : public Address
     public:
     
     AddressRange();
-    AddressRange(const FWObject *root,bool prepopulate);
+    AddressRange(const FWObjectDatabase *root,bool prepopulate);
     AddressRange(AddressRange &);
 
     const InetAddr &getRangeStart() const { return start_address; }
@@ -68,6 +69,11 @@ class AddressRange : public Address
    
     virtual void       fromXML (xmlNodePtr parent) throw(FWException);
     virtual xmlNodePtr toXML   (xmlNodePtr xml_parent_node) throw(FWException);
+
+    virtual bool isPrimaryObject() const { return true; }
+
+    virtual bool dispatchComplexMatch(ObjectMatcher *om, FWObject *obj)
+    { return om->checkComplexMatch(this, obj); }
     
     DECLARE_FWOBJECT_SUBTYPE(AddressRange);
     
