@@ -4,10 +4,10 @@
 
                  Copyright (C) 2000 NetCitadel, LLC
 
-  Author:  Vadim Kurland     vadim@vk.crocodile.org
+  Author:  Vadim Kurland     vadim@fwbuilder.org
            Vadim Zaliva      lord@crocodile.org
 
-  $Id: Logger.cpp 121 2008-07-02 23:00:03Z vadim $
+  $Id: Logger.cpp 234 2008-12-23 06:17:53Z vadim $
 
 
   This program is free software which we release under the GNU General Public
