@@ -4,9 +4,9 @@
 
                  Copyright (C) 2000 NetCitadel, LLC
 
-  Author:  Vadim Kurland     vadim@vk.crocodile.org
+  Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: RuleElement.h 210 2008-10-22 02:18:46Z vadim $
+  $Id: RuleElement.h 264 2009-04-20 16:53:48Z vadim $
 
 
   This program is free software which we release under the GNU General Public
@@ -53,11 +53,11 @@ protected:
 
     bool negation;
     
-    void  _initialize(const FWObject *root);
+    void  _initialize(const FWObjectDatabase *root);
 
 public:
     RuleElement();
-    RuleElement(const FWObject *root,bool prepopulate);
+    RuleElement(const FWObjectDatabase *root,bool prepopulate);
     DECLARE_FWOBJECT_SUBTYPE(RuleElement);
 
     virtual void fromXML(xmlNodePtr parent) throw(FWException);
@@ -95,130 +95,155 @@ class RuleElementSrc : public ObjectGroup, public RuleElement {
     public:
     DECLARE_FWOBJECT_SUBTYPE(RuleElementSrc);
     RuleElementSrc();
-    RuleElementSrc(const FWObject *root,bool prepopulate);
+    RuleElementSrc(const FWObjectDatabase *root,bool prepopulate);
     virtual int getAnyElementId();
+    virtual bool validateChild(FWObject *o);
     virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
+    virtual bool isPrimaryObject() const { return false; }
 };
 
 class RuleElementDst : public ObjectGroup, public RuleElement {
     public:
     DECLARE_FWOBJECT_SUBTYPE(RuleElementDst);
     RuleElementDst();
-    RuleElementDst(const FWObject *root,bool prepopulate);
+    RuleElementDst(const FWObjectDatabase *root,bool prepopulate);
     virtual int getAnyElementId();
+    virtual bool validateChild(FWObject *o);
     virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
+    virtual bool isPrimaryObject() const { return false; }
 };
 
 class RuleElementSrv : public ServiceGroup, public RuleElement {
     public:
     DECLARE_FWOBJECT_SUBTYPE(RuleElementSrv);
     RuleElementSrv();
-    RuleElementSrv(const FWObject *root,bool prepopulate);
+    RuleElementSrv(const FWObjectDatabase *root,bool prepopulate);
     virtual int getAnyElementId();
+    virtual bool validateChild(FWObject *o);
     virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
+    virtual bool isPrimaryObject() const { return false; }
 };
 
 class RuleElementItf : public ObjectGroup, public RuleElement {
     public:
     DECLARE_FWOBJECT_SUBTYPE(RuleElementItf);
     RuleElementItf();
-    RuleElementItf(const FWObject *root,bool prepopulate);
+    RuleElementItf(const FWObjectDatabase *root,bool prepopulate);
     virtual int getAnyElementId();
     virtual bool validateChild(FWObject *o);
     bool checkItfChildOfThisFw(FWObject *o);
     virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
+    virtual bool isPrimaryObject() const { return false; }
 };
 
 class RuleElementInterval : public IntervalGroup, public RuleElement {
     public:
     DECLARE_FWOBJECT_SUBTYPE(RuleElementInterval);
     RuleElementInterval();
-    RuleElementInterval(const FWObject *root,bool prepopulate);
+    RuleElementInterval(const FWObjectDatabase *root,bool prepopulate);
     virtual int getAnyElementId();
+    virtual bool validateChild(FWObject *o);
     virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
+    virtual bool isPrimaryObject() const { return false; }
 };
 
 class RuleElementOSrc : public ObjectGroup, public RuleElement {
     public:
     DECLARE_FWOBJECT_SUBTYPE(RuleElementOSrc);
     RuleElementOSrc();
-    RuleElementOSrc(const FWObject *root,bool prepopulate);
+    RuleElementOSrc(const FWObjectDatabase *root,bool prepopulate);
     virtual int getAnyElementId();
+    virtual bool validateChild(FWObject *o);
     virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
+    virtual bool isPrimaryObject() const { return false; }
 };
 
 class RuleElementODst : public ObjectGroup, public RuleElement {
     public:
     DECLARE_FWOBJECT_SUBTYPE(RuleElementODst);
     RuleElementODst();
-    RuleElementODst(const FWObject *root,bool prepopulate);
+    RuleElementODst(const FWObjectDatabase *root,bool prepopulate);
     virtual int getAnyElementId();
+    virtual bool validateChild(FWObject *o);
     virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
+    virtual bool isPrimaryObject() const { return false; }
 };
 
 class RuleElementOSrv : public ServiceGroup, public RuleElement {
     public:
     DECLARE_FWOBJECT_SUBTYPE(RuleElementOSrv);
     RuleElementOSrv();
-    RuleElementOSrv(const FWObject *root,bool prepopulate);
+    RuleElementOSrv(const FWObjectDatabase *root,bool prepopulate);
     virtual int getAnyElementId();
+    virtual bool validateChild(FWObject *o);
     virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
+    virtual bool isPrimaryObject() const { return false; }
 };
 
 class RuleElementTSrc : public ObjectGroup, public RuleElement {
     public:
     DECLARE_FWOBJECT_SUBTYPE(RuleElementTSrc);
     RuleElementTSrc();
-    RuleElementTSrc(const FWObject *root,bool prepopulate);
+    RuleElementTSrc(const FWObjectDatabase *root,bool prepopulate);
     virtual int getAnyElementId();
+    virtual bool validateChild(FWObject *o);
     virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
+    virtual bool isPrimaryObject() const { return false; }
 };
 
 class RuleElementTDst : public ObjectGroup, public RuleElement {
     public:
     DECLARE_FWOBJECT_SUBTYPE(RuleElementTDst);
     RuleElementTDst();
-    RuleElementTDst(const FWObject *root,bool prepopulate);
+    RuleElementTDst(const FWObjectDatabase *root,bool prepopulate);
     virtual int getAnyElementId();
+    virtual bool validateChild(FWObject *o);
     virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
+    virtual bool isPrimaryObject() const { return false; }
 };
 
 class RuleElementTSrv : public ServiceGroup, public RuleElement {
     public:
     DECLARE_FWOBJECT_SUBTYPE(RuleElementTSrv);
     RuleElementTSrv();
-    RuleElementTSrv(const FWObject *root,bool prepopulate);
+    RuleElementTSrv(const FWObjectDatabase *root,bool prepopulate);
     virtual int getAnyElementId();
+    virtual bool validateChild(FWObject *o);
     virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
+    virtual bool isPrimaryObject() const { return false; }
 };
 
 class RuleElementRDst : public ObjectGroup, public RuleElement {
     public:
     DECLARE_FWOBJECT_SUBTYPE(RuleElementRDst);
     RuleElementRDst();
-    RuleElementRDst(const FWObject *root,bool prepopulate);
+    RuleElementRDst(const FWObjectDatabase *root,bool prepopulate);
     virtual int getAnyElementId();
+    virtual bool validateChild(FWObject *o);
     virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
+    virtual bool isPrimaryObject() const { return false; }
 };
 
 class RuleElementRGtw : public ObjectGroup, public RuleElement {
     public:
     DECLARE_FWOBJECT_SUBTYPE(RuleElementRGtw);
     RuleElementRGtw();
-    RuleElementRGtw(const FWObject *root,bool prepopulate);
+    RuleElementRGtw(const FWObjectDatabase *root,bool prepopulate);
     virtual int getAnyElementId();
     virtual bool validateChild(FWObject *o);
     bool checkSingleIPAdress(FWObject *o);
     bool checkReachableIPAdress(FWObject *o);
     virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
+    virtual bool isPrimaryObject() const { return false; }
 };
 
 class RuleElementRItf : public RuleElementItf {
     public:
     DECLARE_FWOBJECT_SUBTYPE(RuleElementRItf);
     RuleElementRItf();
-    RuleElementRItf(const FWObject *root,bool prepopulate);
+    RuleElementRItf(const FWObjectDatabase *root,bool prepopulate);
     virtual bool validateChild(FWObject *o);
+    virtual bool isPrimaryObject() const { return false; }
 };
 
 }
