@@ -6,22 +6,25 @@
 
   Author:  Tidei Maurizio     <fwbuilder-routing at compal.de>
   
-  Permission is hereby granted, free of charge, to any person obtaining a copy
-  of this software and associated documentation files (the "Software"), to deal
-  in the Software without restriction, including without limitation the rights
-  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
-  of the Software, and to permit persons to whom the Software is furnished to do
-  so, subject to the following conditions: 
+  Permission is hereby granted, free of charge, to any person
+  obtaining a copy of this software and associated documentation files
+  (the "Software"), to deal in the Software without restriction,
+  including without limitation the rights to use, copy, modify, merge,
+  publish, distribute, sublicense, and/or sell copies of the Software,
+  and to permit persons to whom the Software is furnished to do so,
+  subject to the following conditions:
 
-  The above copyright notice and this permission notice shall be included in all
-  copies or substantial portions of the Software. 
+  The above copyright notice and this permission notice shall be
+  included in all copies or substantial portions of the Software.
 
-  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
-  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
-  PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
-  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
-  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
+  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+  SOFTWARE.
 
 */
 
@@ -118,39 +121,39 @@ string RoutingCompiler::debugPrintRule(Rule *r)
 
     string dst, itf, gtw;
    
-    FWObject* obj = itfrel;
-    if (FWReference::cast(itfrel)!=NULL) obj=FWReference::cast( obj)->getPointer();
-    itf=obj->getName();
+    FWObject *obj = FWReference::getObject(itfrel->front());
+    itf = obj->getName();
 
-    if (FWReference::cast(gtwrel)!=NULL) obj=FWReference::cast( obj)->getPointer();
-    gtw=obj->getName();    
+    obj = FWReference::getObject(gtwrel->front());
+    gtw = obj->getName();    
      
     
     int no=0;
     FWObject::iterator i1=dstrel->begin();
-    while ( i1!=dstrel->end()) {
-
+    while ( i1!=dstrel->end())
+    {
         str << endl;
 
-        dst=" ";
+        dst = " ";
 
-        if (i1!=dstrel->end()) {
-            FWObject *o=*i1;
-            if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
-            dst=o->getName();
+        if (i1!=dstrel->end())
+        {
+            FWObject *o = FWReference::getObject(*i1);
+            dst = o->getName();
         }
 
         int w=0;
-        if (no==0) {
+        if (no==0)
+        {
             str << rule->getLabel();
-            w=rule->getLabel().length();
+            w = rule->getLabel().length();
         }
         
         str <<  setw(10-w)  << setfill(' ') << " ";
 
-        str <<  setw(18) << setfill(' ') << dst.c_str();
-        str <<  setw(18) << setfill(' ') << itf.c_str();
-        str <<  setw(12) << setfill(' ') << gtw.c_str();
+        str <<  setw(18) << setfill(' ') << dst.c_str() << " ";
+        str <<  setw(18) << setfill(' ') << itf.c_str() << " ";
+        str <<  setw(18) << setfill(' ') << gtw.c_str() << " ";
         str <<  setw(18) << setfill(' ') << " ";
 
         ++no;
@@ -160,23 +163,19 @@ string RoutingCompiler::debugPrintRule(Rule *r)
     return str.str();
 }
 
-
 bool RoutingCompiler::ExpandMultipleAddresses::processNext()
 {
     RoutingRule *rule=getNext(); if (rule==NULL) return false;
-
     tmp_queue.push_back(rule);
 
-    RuleElementRDst *dst=rule->getRDst();    assert(dst);
-    RuleElementRGtw *gtw=rule->getRGtw();    assert(gtw);
-    
-    compiler->_expandAddr(rule,dst);
-    compiler->_expandAddr(rule,gtw);
+    RuleElementRDst *dst = rule->getRDst();    assert(dst);
+    RuleElementRGtw *gtw = rule->getRGtw();    assert(gtw);
 
+    compiler->_expandAddr(rule, dst);
+    compiler->_expandAddr(rule, gtw);
     return true;
 }
 
-
 bool RoutingCompiler::ConvertToAtomicForDST::processNext()
 {
     RoutingRule *rule=getNext(); if (rule==NULL) return false;
@@ -185,10 +184,9 @@ bool RoutingCompiler::ConvertToAtomicForDST::processNext()
     RuleElementRDst *dst=rule->getRDst();    assert(dst);
 
 
-    for (FWObject::iterator it=dst->begin(); it!=dst->end(); ++it) {
-
-        RoutingRule *r = RoutingRule::cast(
-            compiler->dbcopy->create(RoutingRule::TYPENAME) );
+    for (FWObject::iterator it=dst->begin(); it!=dst->end(); ++it)
+    {
+        RoutingRule *r = compiler->dbcopy->createRoutingRule();
         r->duplicate(rule);
         compiler->temp_ruleset->add(r);
 
@@ -211,13 +209,10 @@ bool RoutingCompiler::ConvertToAtomicForDST::processNext()
 bool RoutingCompiler::ExpandGroups::processNext()
 {
     RoutingRule *rule=getNext(); if (rule==NULL) return false;
-
     tmp_queue.push_back(rule);
 
     RuleElementRDst *dst=rule->getRDst();   assert(dst);
-    
     compiler->expandGroupsInRuleElement(dst);
-
     return true;
 }
 
@@ -225,7 +220,6 @@ bool RoutingCompiler::ExpandGroups::processNext()
 bool RoutingCompiler::emptyRDstAndRItf::processNext()
 {
     RoutingRule *rule=getNext(); if (rule==NULL) return false;
-    
     tmp_queue.push_back(rule);
     
     
@@ -233,10 +227,11 @@ bool RoutingCompiler::emptyRDstAndRItf::processNext()
     RuleElementRItf *itfrel=rule->getRItf();
 
     if ( (FWReference::cast(itfrel->front())->getPointer())->getName()=="Any" &&\
-         (FWReference::cast(gtwrel->front())->getPointer())->getName()=="Any") {
-        
+         (FWReference::cast(gtwrel->front())->getPointer())->getName()=="Any")
+    {
         string msg;
-        msg = "Gateway and interface are both empty in the rule " + rule->getLabel();
+        msg = "Gateway and interface are both empty in the rule " +
+            rule->getLabel();
         compiler->abort( msg.c_str() );
     }
 
@@ -247,15 +242,14 @@ bool RoutingCompiler::emptyRDstAndRItf::processNext()
 bool RoutingCompiler::singleAdressInRGtw::processNext()
 {
     RoutingRule *rule=getNext(); if (rule==NULL) return false;
-    
     tmp_queue.push_back(rule);
     
-    
     RuleElementRGtw *gtwrel=rule->getRGtw();
-    FWObject *o = FWReference::cast(gtwrel->front())->getPointer();
+
+    FWObject *o = FWReference::getObject(gtwrel->front());
               
-    if( gtwrel->checkSingleIPAdress(o) == false) {
-        
+    if( gtwrel->checkSingleIPAdress(o) == false)
+    {       
         string msg;
         msg = "The object \"" + o->getName() + "\" used as gateway in the routing rule "\
             + rule->getLabel() + " has more than one interface resp. IP adress!";
@@ -501,31 +495,15 @@ bool RoutingCompiler::rItfChildOfFw::processNext()
     RuleElementRItf *itfrel = rule->getRItf();
     FWObject *o = FWReference::cast(itfrel->front())->getPointer();
 
-    /*
-     * check if an object 'o' is a child of the firewall being
-     * processed
-     */
-    FWObject* o_tmp = o;
-    bool same_fw = false;
-    while (o_tmp)
-    {
-        if (Firewall::cast(o_tmp) && o_tmp->getId()==compiler->fw->getId())
-        {
-            same_fw = true;
-            break;
-        }
-        o_tmp = o_tmp->getParent();
-    }
-              
-    if (!same_fw)
+    if (!o->isChildOf(compiler->fw))
     {
         string msg;
         msg = "The object \"" + o->getName() + 
             "\" used as interface in the routing rule " +
             rule->getLabel() +
-            " is not a child of the firewall the rule belongs to!";
+            " is not a child of the firewall the rule belongs to";
         compiler->abort( msg.c_str() );
-    } 
+    }
     return true;
 }
 
@@ -567,8 +545,10 @@ bool RoutingCompiler::competingRules::processNext()
                 ///std::cout << "SAME METRIC" << std::endl;
                 
                 string msg;
-                msg = "The routing rules " + gtwitf_it->second.second + " and " + rule->getLabel() +
-                      "are identical,\n skipping the second. Please delete one of them to avoid this warning!";
+                msg = "The routing rules " + gtwitf_it->second.second +
+                    " and " + rule->getLabel() +
+                    " are identical, skipping the second. " +
+                    "Please delete one of them to avoid this warning!";
                 compiler->warning( msg.c_str() );
             
             } else {
@@ -577,8 +557,10 @@ bool RoutingCompiler::competingRules::processNext()
                 ///std::cout << "DIFFERENT METRIC" << std::endl;
                 
                 string msg;
-                msg = "The routing rules " + gtwitf_it->second.second + " and " + rule->getLabel() +
-                      "are identical\n except the metric, please delete one of them!";
+                msg = "The routing rules " + gtwitf_it->second.second +
+                    " and " + rule->getLabel() +
+                    " are identical except for the metric, " +
+                    "please delete one of them!";
                 compiler->abort( msg.c_str() );
             }
         
@@ -592,8 +574,6 @@ bool RoutingCompiler::competingRules::processNext()
                 //               iterate all gtwitf combis in the map dest_it->second and search for the current metric
                 
                 // ... but has the same metric => what route should I use for this destination? => abort
-                
-                
                     
                 string msg;
                 msg = "The routing rules " + gtwitf_it->second.second + " and " + rule->getLabel() + " have the same destination,\
